@@ -36,6 +36,16 @@ os_win_opts = [
                help='The default amount of seconds to wait when stopping '
                     'pending WMI jobs. Setting this value to 0 will '
                     'disable the timeout.'),
+    cfg.IntOpt('connect_cluster_timeout',
+               default=0,
+               help='The amount of time to wait for the Failover Cluster '
+                    'service to be available.'),
+    cfg.IntOpt('file_in_use_timeout',
+               default=15,
+               help='The amount of seconds to wait for in-use files when '
+                    'performing moves or deletions. This can help mitigate '
+                    'issues occurring due to Hyper-V locks or even 3rd party '
+                    'backup tools.'),
 ]
 
 CONF = cfg.CONF
