@@ -18,8 +18,8 @@ Unit tests for the Hyper-V utils factory.
 """
 
 import inspect
+from unittest import mock
 
-import mock
 from oslo_config import cfg
 from oslo_utils import importutils
 
