@@ -314,7 +314,8 @@ class VMUtils(baseutils.BaseUtilsVirt):
                   configuration_root_dir=None, snapshot_dir=None,
                   host_shutdown_action=None, vnuma_enabled=None,
                   snapshot_type=None,
-                  is_planned_vm=False):
+                  is_planned_vm=False,
+                  chassis_asset_tag=None):
         vmsetting = self._lookup_vm_check(vm_name, for_update=True)
 
         if host_shutdown_action:
@@ -338,6 +339,9 @@ class VMUtils(baseutils.BaseUtilsVirt):
         if snapshot_type:
             self._set_vm_snapshot_type(vmsetting, snapshot_type)
 
+        if chassis_asset_tag:
+            vmsetting.ChassisAssetTag = chassis_asset_tag
+
         self._modify_virtual_system(vmsetting)
 
     def check_admin_permissions(self):
@@ -658,19 +662,26 @@ class VMUtils(baseutils.BaseUtilsVirt):
         return [nic.ElementName for nic in nics]
 
     def soft_shutdown_vm(self, vm_name):
-        vm = self._lookup_vm_check(vm_name, as_vssd=False)
-        shutdown_component = self._conn.Msvm_ShutdownComponent(
-            SystemName=vm.Name)
+        try:
+            vm = self._lookup_vm_check(vm_name, as_vssd=False)
+            shutdown_component = self._conn.Msvm_ShutdownComponent(
+                SystemName=vm.Name)
 
-        if not shutdown_component:
-            # If no shutdown_component is found, it means the VM is already
-            # in a shutdown state.
-            return
+            if not shutdown_component:
+                # If no shutdown_component is found, it means the VM is already
+                # in a shutdown state.
+                return
 
-        reason = 'Soft shutdown requested by OpenStack Nova.'
-        (ret_val, ) = shutdown_component[0].InitiateShutdown(Force=False,
-                                                             Reason=reason)
-        self._jobutils.check_ret_val(ret_val, None)
+            reason = 'Soft shutdown requested by OpenStack Nova.'
+            (ret_val, ) = shutdown_component[0].InitiateShutdown(Force=False,
+                                                                 Reason=reason)
+            self._jobutils.check_ret_val(ret_val, None)
+        except exceptions.x_wmi as ex:
+            # This operation is expected to fail while the instance is booting.
+            # In some cases, InitiateShutdown immediately throws an error
+            # instead of returning an asynchronous job reference.
+            msg = _("Soft shutdown failed. VM name: %s. Error: %s.")
+            raise exceptions.HyperVException(msg % (vm_name, ex))
 
     @_utils.retry_decorator(exceptions=exceptions.WMIJobFailed)
     def set_vm_state(self, vm_name, req_state):
